// ****************************************************************************
//                               Copyright
//                          Robert Tyrakowski
//                             r.ty@rbty.de
//                                 DK7NT
//                              24.04.2005
//*****************************************************************************
//
// This software is an example how to control the Nokia 7560 color screen by
// a LP2106. One can use it for personal projects, just like it is, as long the
// Copyright is not removed . I can not guarantee that the code is without
// errors. I'm not responsible for any problem which occur using this code.
//
// The source code is not optimised for C, but optimzed for debugging. It's
// written for the Imagecraft C-Compiler ICCARM7. (www.imagecraft.com)
//
//          ******  Commercial use is not permitted  ******
//
// **** I'll not support any user or answer any question about the code ****
//
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
// Zu erledigen:
//
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                    LCD7650 Color Display Controller
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  Target: LPC2106
//  Clock: 14.7456 MHz * 4 = ~60 MHz
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 const unsigned char teststring0[]  = "DK7NT (C) 2005";
 const unsigned char teststring1[] = "  V0.1";
 const unsigned char tbm[] = "DK7NT\0";

#define simulation 0

// LPC2106 IO

// P0.00	s	TXD0	: 13
// P0.01	s	RXD0	: 14
// P0.02	s	SLC		: 18
// P0.03	s	SDA		: 21

// P0.04	s	SCK		: 22
// P0.05	s	MISO	: 23
// P0.06	s	MOSI	: 24
// P0.07	s	SSEL	: 28

// P0.08	ss	LEDPWM	: 29
// P0.09	o	RESET 	: 30
// P0.10	o	OELCD 	: 35
// P0.11	o	SEL0 	: 36

// P0.12	o	SEL1	: 37
// P0.13	o 	test	: 41
// P0.14	s  int1		: 44
// P0.15	o  A0   	: 45

// P0.16	s  int0		: 46
// P0.17	o	WR    	: 47
// P0.18	o	RD		: 48
// P0.19	o	D0		: 1

// P0.20	o	D1		: 2
// P0.21	o	D2		: 3
// P0.22	o	D3 		: 32
// P0.23	o	D4 		: 33

// P0.24	o	D5 		: 34
// P0.25	o	D6 		: 38
// P0.26	o	D7 		: 39
// P0.27	s	TRST  	: 8

// P0.28	s	TMS 	: 9
// P0.29	s	TCK 	: 10
// P0.30	s	TDI 	: 15
// P0.31	s	TDO 	: 16

//-----------------------------------------------------------------------------
// 0 x------>
// y +----------------+
// | |                |
// | |                |
// V |                |
//   |                |
//   |                |
//   |                |
//   |                |
//   |                |
//   |                |
//   +----------------+
//
//          /---\
//          | O |
//          \---/
//
//-----------------------------------------------------------------------------

#include <LPC210x.h>
#include <arm_macros.h>
#include <LCD7650.h>

#include <stdio.h>
#include <math.h>
#include <arial_bold_14.h>
#include <corsiva_12.h>
#include <Arial_24.h>

//#define image pisairfan
//#include <pisairfan.c>
#include <tb.c>
#define image tb

# define transmission_start 0xAA
# define transmission_start1 0x55
# define transmission_end 0xF2

//-----------------------------------------------------------------------------
// Variables

unsigned char backlight; // contains current backlight intensity (0..10=0..100%)

unsigned int Coordx;	 		   // current x
unsigned int Coordy;	 			// current y
unsigned int Coordx_save;	   // current x
unsigned int Coordy_save;	 	// current y

unsigned int CoordRAM_index;	// RAM Address
unsigned char Coordidx;			// idex
unsigned long b0,b1,b2;			// color translated values

// LCD Image in RAM (+4 for safety)
unsigned long LCD_RAM_Image[((LCD_x_max+1) * (LCD_y_max+1)/2 *3 /4)+ 4];
// lenght of Image buffer in RAM
unsigned int Lcd_Image_Length = (((LCD_x_max+1) * (LCD_y_max+1))/2*3)/4;

//Uart0 buffer
unsigned char Rx_Buff[64];
unsigned char rx_rdy;            // Transmission status: 0,1,2,3
unsigned char rx_in;			 		// Number of entries in RX_Buff
unsigned char lastlen, rs232_length;		// message length

// Timer 0
unsigned int timeval;

//
unsigned char process;
unsigned char command;

unsigned long rocker_value, key_value;

// Cursor
unsigned char cursor, cur_onoff;
unsigned char cur_type,cur_blink,cur_color,cur_bgcolor,cur_fix, cur_landscape;
unsigned long cur_cnt;
unsigned long cur_save[128];
unsigned char cur_onoffcont;
unsigned long CoordRAM_index_save;

# define cur_blinkfreq 300
# define debounce 5

// font information
 const unsigned char* currentfont;

 unsigned char loaded_font[6500];

// ----------------------------------------------------------------------------
// Prototypes
// ----------------------------------------------------------------------------
void wait(unsigned long x);
void Write_LCD(unsigned long rr, unsigned char a0);
void Image2lcd(unsigned long *LCD_RAM_Image,unsigned int Lcd_Image_Length,unsigned long x);
void Image2lcdasm(void);
void GotoXY(unsigned char x, unsigned char y);
void SetDot(unsigned char x, unsigned char y, unsigned short color);
void SetDotDot(unsigned char x, unsigned char y,unsigned char lwidth, unsigned short color);
void FillRect(unsigned char x, unsigned char y, unsigned char width, unsigned char height, unsigned char lwith, unsigned short color);
unsigned long ReadRocker(void);
unsigned long ReadKey(void);
void Select_Source(unsigned char source);
void Cursor_WR_Underscore_upright(void);
void Cursor_WR_Underscore_landscape(void);
void Save_Current_Coord(void);
void Restore_Current_Coord(void);

//=============================================================================
// Interrupts
//=============================================================================

#pragma interrupt_handler ext0_isr
void ext0_isr(void)
{
   rocker_value = ReadRocker() | 0x80000000;
	Select_Source(0);
   SCB_EXTINT |= 0x01; // Clear EINT0 interrupt flag
   VICVectAddr = 0;    // Acknowledge Interrupt
	VICIntEnClear = 0x4000;				  // block extint0
}

#pragma interrupt_handler ext1_isr
void ext1_isr(void)
{
   key_value = ReadKey() | 0x80000000;
	Select_Source(0);
   SCB_EXTINT |= 0x02; // Clear EINT1 interrupt flag
   VICVectAddr = 0;    // Acknowledge Interrupt
}

/*
#pragma interrupt_handler timer0_isr
void timer0_isr(void)
{
	timeval--;
	T0_IR = 1; 				  // Clear interrupt flag
   VICVectAddr = 0; 		  // Acknowledge Interrupt
}
*/

#pragma interrupt_handler uart0_isr
void uart0_isr(void)
{
   unsigned char data;
   data = UART0_IIR; //read interrupt id to clear flag
   data= UART0_RBR;
   if (rx_rdy) {
      if (rx_rdy == 1) {
	      if (data == transmission_start1) {
		      rx_rdy = 2;
		   }
		   else {
		      rx_rdy = 0;                 // start again
		   }
	   }
	   else {
         if (rx_rdy == 2) {
			   rs232_length = data;			 // filter message length
				lastlen = data;
				rx_rdy=3;
		   }
			else {
				if (rx_rdy == 3 && rs232_length--) {
               Rx_Buff[rx_in++] = data ;// store message data
				}
				else {
				   if (data == transmission_end) {
		            rx_rdy = 5;           // transmission end
					}
					else {
					   rx_rdy = 6;				 // ready, but error in message
				   }
		      }
	      }
		}
   }
   else {
      if (data == transmission_start) {
	      rx_rdy = 1;
	   }
   }
  VICVectAddr = 0x00000000;             // clear this interrupt from the VIC
}

//=============================================================================
// System Initialization
//=============================================================================

void timer0_init()
{
 //Clock frequency=1 mS
 //Channel 0 = 1 Hz
 //Channel 1 = 1 Hz
 //Channel 2 = 1 Hz
 //Channel 3 = 1 Hz
 T0_TCR=0x00000001;  // Timer0 Enable
 T0_TC=0x00000000;
 T0_PR=0x0000E666;  // 1ms
 T0_PC=0x00000000;
 T0_MCR=0x00000000;  // Interrupt and Reset on MR0
 T0_MR0=0x000003E8;  // Compare-hit tick
 T0_MR1=0x000003E8;
 T0_MR2=0x000003E8;
 T0_MR3=0x000003E8;
 T0_CCR=0x00000000;
 T0_EMR=0x00000000;
}

void port_init()
{
   PCB_PINSEL0 = PINSEL0;
   PCB_PINSEL1 = PINSEL1;

   GPIO_IODIR = IODIR;
   GPIO_IOCLR = IOCLR;
   GPIO_IOSET = IOSET;

   LCD_OELCD_high;
}

void uart0_init()
{
 //Baud rate=38400 (0%)
 UART0_IER=0x00000001;
 UART0_FCR=0x00000000;
 UART0_LCR=0x00000083;
 UART0_DLL=0x00000060&0x00ff;
 UART0_DLM=0x00000060>>8;
 UART0_LCR=0x00000003;
 rx_in = 0;
}

void init_pll()
{
#if simulation
  ;
#else
	// --- enable and connect the PLL (Phase Locked Loop) ---
	// set multiplier and divider
	SCB_PLLCFG = MSEL | (1<<PSEL1) | (0<<PSEL0);
	SCB_PLLCON = (1<<PLLE);		   	 			// enable PLL
	SCB_PLLFEED = PLL_FEED1;					// feed sequence
	SCB_PLLFEED = PLL_FEED2;					// feed sequence
	// wait for PLL lock (PLOCK bit is set if locked)
	while (!(SCB_PLLSTAT & (1<<PLOCK)));
	SCB_PLLCON = (1<<PLLE) | (1<<PLLC); 	    // connect (and enable) PLL
	SCB_PLLFEED = PLL_FEED1;		   	   	    // feed sequence
	SCB_PLLFEED = PLL_FEED2;                    // feed sequence
#endif
}

void pll_speed(unsigned char divider)
{
	SCB_PLLCFG = divider | (1<<PSEL1) | (0<<PSEL0);
	SCB_PLLCON = (1<<PLLE) | (1<<PLLC); 	    // connect (and enable) PLL
	SCB_PLLFEED = PLL_FEED1;					// feed sequence
	SCB_PLLFEED = PLL_FEED2;					// feed sequence
}

void pwm_init()
{
   PWM_TCR=0; 		   	   //disable pwm before setup
   PWM_TC=0; 			  	   //clear counter
   PWM_PR=0x000000000;	   //set prescale value
   PWM_PC=0;				   //clear prescale counter
   PWM_MCR=0x00000000;
   PWM_PCR=0x00001000;
   PWM_MR0=pwm_periode;	   //around 1 ms periode
   PWM_MR1=0x00000000;
   PWM_MR2=0x00000000;
   PWM_MR3=0x00000000;
   PWM_MR4=pwm_periode/2;  //50 % duty cycle
   backlight = 5;
   PWM_MR5=0x00000000;
   PWM_MR6=0x00000000;
   PWM_LER=0x7F;			   //set pwm latch enable to load values
   PWM_TCR=0x00000009;
}

void watchdog_init()
{
   WD_WDTC=0x000000FF;
   WD_WDMOD=0x00000000;
 //start watchdog
   WD_WDFEED=0xaa;
   WD_WDFEED=0x55;
}

void init_devices()
{
   __DISABLE_INTERRUPT();
   init_pll();
   watchdog_init();
   port_init();
   pwm_init();

   SCB_EXTWAKE=0x00000000;
   SCB_VPBDIV = 0x01; ; //peripheral clock divider
   timer0_init();
   uart0_init();
   MAM_TIM = 0x2;
   MAM_CR = 0x02;

//select interrupts
   VICIntSelect=0x00000000;						// select FIQ (no FIQ)
   VICVectAddr0 = (unsigned)(uart0_isr);    	// address of the ISR
//   VICVectAddr1 = (unsigned)(timer0_isr);    // address of the ISR
   VICVectAddr2 = (unsigned)(ext0_isr);    	// address of the ISR
   VICVectAddr3 = (unsigned)(ext1_isr);    	// address of the ISR
   VICVectCntl0 = 0x20 | 6;		  // UART0  #6
//   VICVectCntl1 = 0x20 | 4;		  // Timer0 #4
   VICVectCntl2 = 0x20 | 14;		  // ext0   #14
   VICVectCntl3 = 0x20 | 15;		  // ext1   #15
//   VICIntEnable= 0x00000040 | 0x00000010 | 0x00008000 | 0x00004000;// enable interrups
   VICIntEnable= 0x00000040 | 0x00008000 | 0x00004000;// enable interrups
	SCB_EXTINT |= 0x03; // clear intflag ext0/1
   __ENABLE_INTERRUPT();
}

//-----------------------------------------------------------------------------
// copy image from Flash to RAM_Image
//-----------------------------------------------------------------------------
//           0  1  2
// input is rg br gb
//
// for 8 bits:
//             x   y   z
//-----------------------
//bit 31..28  r2  b4  g7
//bit 27..24  g2  r5  b7
//bit 23..20  g1  r4  b6
//bit 19..16  b1  g4  r7
//bit 15..12  b0  g3  r6
//bit 11.. 8  r1  b3  g6
//bit  7.. 4  r0  b2  g5
//bit  3.. 0  g0  r3  b5

void Copy_Image_FR(void)
{
unsigned int i,j;
unsigned long x,y,z;
   i=0;
   j=0;
//
   do  {
      x  = (long)(image[i][0]);         //rg -
      x |= (long)(image[i][1]<<8);      //br
      x |= (long)(image[i++][2]<<16);   //gb
      x |= (long)(image[i][0]<<24);     //rg
      y  = (long)(image[i][1]);         //br -
      y |= (long)(image[i++][2]<<8);	 //gb
      y |= (long)(image[i][0]<<16);     //rg
      y |= (long)(image[i][1]<<24);     //br
      z  = (long)(image[i++][2]);	    //gb -
      z |= (long)(image[i][0]<<8);	    //rg
      z |= (long)(image[i][1]<<16);	    //br
      z |= (long)(image[i++][2]<<24);	 //gb
      LCD_RAM_Image[j++] = x;
	   LCD_RAM_Image[j++] = y;
	   LCD_RAM_Image[j++] = z;
	   }
   while (j < Lcd_Image_Length);
}


//------------------------------------------------------
// Backlight setup : 0    = off
//                   1..9 = 10 .. 90 %
//                   10   = 100%
void Back_Light(unsigned char value)
{
float x;
   if (value) {
      if (value > 10) {
	    value = 10;
	  }
     x =100/((float)(value)*10);
	  x = pwm_periode/x;
	  PWM_MR4=(long)(x);    // duty cycle
   }
   else {
      PWM_MR4=0;
   }
   backlight = value;
   PWM_LER=0x7F;	//set pwm latch enable to load values
}

//--------------------------------------------
// Backlight up/down
//    0 = down;
//    1 = up;
//
void Back_Light_UD(unsigned char ud)
{
float x;
   if (ud) {				// darker
      if ( backlight < 10) {
	    backlight++;
	  }
	  else {
	    backlight = 10;
      }
   }
   else {		  	   	   // brighter
      if (backlight) {
	    backlight--;
	  }
	  else {
	    backlight = 0;
      }
   }
   Back_Light(backlight);
}


/*
void wait(unsigned long x)
{
   T0_TCR=0x00000002;      // Timer0 Reset
	timeval=x;
	T0_IR = 1; 				  // Clear interrupt flag
   VICVectAddr = 0; 		  // Acknowledge Interrupt
   T0_TCR=0x00000001;     // Timer0 Enable
   test_high;
	while(timeval) {;}
   test_low;
   T0_TCR=0x00000002;      // Timer0 Reset
}
*/
#if simulation

void wait(unsigned long x)
{
  ;
}

#else
void wait(unsigned long x)
{
unsigned long y,z;
unsigned char rdy;
	timeval=x;
   y = T0_TC;
	test_high;
	rdy = 0;
	while(!rdy) {
	   z = T0_TC;
		if (z < y) {
		   if ((0xffffffff-z)+y > x) {
			   rdy = 1;
		   }
		}
		else {
		   if (z-y > x) {
			   rdy = 1;
		   }
		}
	}
   test_low;
}
#endif

//-----------------------------------------------------------------------------
// 0 : LCD write
// 1 : Rocker and Setup read
// 2 : Key and Status read
// 3 : LCD read
void Select_Source(unsigned char source)
{
   GPIO_IOSET = LCD_OELCD;
	switch (source)
   {
      case 0:GPIO_IOCLR = LCD_Sel0 | LCD_Sel1;   // for write to LCD
			 	 GPIO_IODIR = IODIR;
	  		    GPIO_IOCLR = LCD_OELCD;
	  	   	 break;
	   case 1:GPIO_IODIR = IODIR1;
	  	       GPIO_IOSET = LCD_Sel0;	 		     // read Rock and Setup
	  	       GPIO_IOCLR = LCD_Sel1;
	  	       break;
	   case 2:GPIO_IODIR = IODIR1;
	          GPIO_IOSET = LCD_Sel1;			     // read Key
	  	     	 GPIO_IOCLR = LCD_Sel0;
	  	   	 break;
	   default:GPIO_IODIR = IODIR1;
	           GPIO_IOSET = LCD_Sel0 | LCD_Sel1; // read LCD
	  		     GPIO_IOCLR = LCD_OELCD;
   }
}

void Write_LCD(unsigned long rr, unsigned char a0)
{
 	if (a0) LCD_a0_high;
	else LCD_a0_low;
      DataClear_rr;
 	DataOut_rr;
	LCD_wr_low;
   LCD_wr_high;
}

/*
unsigned char Read_LCD(unsigned char a0)
{
unsigned char value;
 	if (a0) LCD_a0_high;
	else LCD_a0_low;
// 	P4DIR=0x00;
	LCD_rd_low;
// 	value = DataIn();
	LCD_rd_high;
//	P4DIR=0xFF;
//	return(value);
    return(0);
}
*/
//=============================================================================
// LCD Functions
//=============================================================================
//-------------------------------------------------------
// DISON
//
// Controlls display operative state. Keep 100ms distance from LF2_SLEEPOUT
// and LF2_DisplayOff
#define L2F_DisplayOn	          Write_LCD(0xAF,0x00)

//-------------------------------------------------------
// DISNOR
//
// Sets whole display to positiv image.
#define L2F_DisplayNormal		  Write_LCD(0xA6,0x00)

//-------------------------------------------------------
// DISINV
//
// Sets whole display to negative image.
#define L2F_DisplayInvers         Write_LCD(0xA7,0x00)

//-------------------------------------------------------
// DISOFF
//
// Sets display off . Keep 80ms distance from LF2_DisplayOff
#define L2F_DisplayOff   		  Write_LCD(0xAE,0x00)

//-----------------------------------------------------------------------------
// Translate colors for Image (one color for the hole screen)
// color  : unsigned short 0rgb
// for 8 bits:
//           b0 b1 b2
//---------------------
//bit 31..28  r  b  g
//bit 27..24  g  r  b
//bit 23..20  g  r  b
//bit 19..16  b  g  r
//bit 15..12  b  g  r
//bit 11.. 8  r  b  g
//bit  7.. 4  r  b  g
//bit  3.. 0  g  r  b

void Translate_8Bit(short color)
{
unsigned long r, g, b;
   r = (long)((color & 0x0F00)>>8);
   g = (long)((color & 0x00F0)>>4);
   b = (long)(color & 0x000F);

   b0 = g | r<<4 | r<<8 | b<<12 | b<<16 | g<<20 | g<<24 | r<<28;
   b1 = r | b<<4 | b<<8 | g<<12 | g<<16 | r<<20 | r<<24 | b<<28;
   b2 = b | g<<4 | g<<8 | r<<12 | r<<16 | b<<20 | b<<24 | g<<28;
}

//-------------------------------------------------------------------------
// L2F_Background , fills background with color
// r is red, g is green , b is blue for a single pb !!!
// short coclor = 0rbg

void L2F_Background(short color)
{
unsigned int i = 0;
unsigned int cnt;
   Translate_8Bit(color);
   cnt = ((LCD_x_max+1) * (LCD_y_max+1)/2 *3 /4);
   do  {

	  LCD_RAM_Image[i++]= b0 ;
	  LCD_RAM_Image[i++]= b1 ;
	  LCD_RAM_Image[i++]= b2 ;
   }
   while(i < cnt);
   Image2lcdasm();
//   wait (1000);  	// 1 Sekunde
}

//-----------------------------------------------------------------------------
// ImageToLCD
// Muss spter im RAM mit full speed laufen
//
//#pragma abs_address:0x40000000
void Image2lcdasm(void)
{
unsigned long lcddat;
   lcddat = LCD_DATA;
   Select_Source(0);      // write to LCD
   Write_LCD(0x5c,0x00);
   LCD_a0_high;
   pll_speed(4);		 // divider 5 (5-1) - out of spec!!!
 //  test_high;
   Image2lcd(LCD_RAM_Image,Lcd_Image_Length,lcddat);
   pll_speed(3);		 // divider 4 (4-1) - within spec. !!
 //  test_low;
   Write_LCD(0xC6,0x00);
   Write_LCD(0x4D,0x01);
   L2F_DisplayOn;
}

//#pragma end_abs_address

//*****************************************************************************

void DrawLine(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2, unsigned char lwidth, unsigned short color) {
	unsigned char length, i, y, yAlt, xTmp, yTmp;
	unsigned int m;

	//
	// vertical line
	//
	if(x1 == x2) {
		// x1|y1 must be the upper point
		if(y1 > y2) {
			yTmp = y1;
			y1 = y2;
			y2 = yTmp;
		}
		DrawVertLine(x1, y1, y2-y1, lwidth, color);
	//
	// horizontal line
	//
	} else if(y1 == y2) {
		// x1|y1 must be the left point
		if(x1 > x2) {
			xTmp = x1;
			x1 = x2;
			x2 = xTmp;
		}
		DrawHoriLine(x1, y1, x2-x1, lwidth, color);
	//
	// schiefe line :)
	//
	} else {
		// angle >= 45
		if((y2-y1) >= (x2-x1) || (y1-y2) >= (x2-x1)) {
			// x1 must be smaller than x2
			if(x1 > x2) {
				xTmp = x1;
				yTmp = y1;
				x1 = x2;
				y1 = y2;
				x2 = xTmp;
				y2 = yTmp;
			}

			length = x2-x1;		// not really the length :)
			m = ((y2-y1)*200)/length;
			yAlt = y1;

			for(i=0; i<=length; i++) {
				y = ((m*i)/200)+y1;

				if((m*i)%200 >= 100)
					y++;
				else if((m*i)%200 <= -100)
					y--;

				DrawLine(x1+i, yAlt, x1+i, y, lwidth, color);

				if(length <= (y2-y1) && y1 < y2)
					yAlt = y+1;
				else if(length <= (y1-y2) && y1 > y2)
					yAlt = y-1;
				else
					yAlt = y;
			}
		// angle < 45
		} else {
			// y1 must be smaller than y2
			if(y1 > y2) {
				xTmp = x1;
				yTmp = y1;
				x1 = x2;
				y1 = y2;
				x2 = xTmp;
				y2 = yTmp;
			}
			length = y2-y1;
			m = ((x2-x1)*200)/length;
			yAlt = x1;

			for(i=0; i<=length; i++) {
				y = ((m*i)/200)+x1;

				if((m*i)%200 >= 100)
					y++;
				else if((m*i)%200 <= -100)
					y--;

				DrawLine(yAlt, y1+i, y, y1+i, lwidth, color);
				if(length <= (x2-x1) && x1 < x2)
					yAlt = y+1;
				else if(length <= (x1-x2) && x1 > x2)
					yAlt = y-1;
				else
					yAlt = y;
			}
		}
	}
}

void DrawRect(unsigned char x, unsigned char y, unsigned char width, unsigned char height, unsigned char lwidth,unsigned short color)
{
	DrawHoriLine(x, y, width,  lwidth, color);				// top
	DrawHoriLine(x, y+height, width, lwidth, color);		// bottom
	DrawVertLine(x, y, height, lwidth, color);				// left
	DrawVertLine(x+width, y, height, lwidth, color);		// right
}

// ----------------------------------------------------------------------------
// rectangular with round corners
//-----------------------------------------------------------------------------
void DrawRoundRect(unsigned char x, unsigned char y, unsigned char width, unsigned char height, unsigned char radius,unsigned char lwidth, unsigned short color) {
  	int tSwitch, x1 = 0, y1 = (int)(radius);
  	tSwitch = 3 - 2 * radius;

	while (x1 <= y1) {
	    SetDotDot(x+radius - x1, y+radius - y1, lwidth, color);
	    SetDotDot(x+radius - y1, y+radius - x1, lwidth, color);

	    SetDotDot(x+width-radius + x1, y+radius - y1, lwidth, color);
	    SetDotDot(x+width-radius + y1, y+radius - x1, lwidth, color);

	    SetDotDot(x+width-radius + x1, y+height-radius + y1, lwidth, color);
	    SetDotDot(x+width-radius + y1, y+height-radius + x1, lwidth, color);

	    SetDotDot(x+radius - x1, y+height-radius + y1, lwidth, color);
	    SetDotDot(x+radius - y1, y+height-radius + x1, lwidth, color);

	    if (tSwitch < 0) {
	    	tSwitch += (4 * x1 + 6);
	    } else {
	    	tSwitch += (4 * (x1 - y1) + 10);
	    	y1--;
	    }
	    x1++;
	}

	DrawHoriLine(x+radius, y, width-(2*radius), lwidth, color);			// top
	DrawHoriLine(x+radius, y+height, width-(2*radius), lwidth, color);	// bottom
	DrawVertLine(x, y+radius, height-(2*radius), lwidth, color);		// left
	DrawVertLine(x+width, y+radius, height-(2*radius), lwidth, color);	// right
}

//
// Hardware-Functions
//

//----------------------------------------------------------------------------
// Fill Rectangual with color
// short color = 0x0rgb
//

void FillRect(unsigned char x, unsigned char y, unsigned char width, unsigned char height, unsigned char lwidth, unsigned short color)
{
unsigned char i,j ;
   if (lwidth) {
		x -= lwidth;
		y -= lwidth;
		lwidth *= 2;
	   for (j=0;j<=height+lwidth;j++) {
         for(i=0; i<=width+lwidth; i++) {
            SetDot(x+i,y+j,color);
         }
      }
	}
	else {
	   for (j=0;j<=height;j++) {
         for(i=0; i<=width; i++) {
            SetDot(x+i,y+j,color);
         }
      }
   }
}

//=============================================================================
// Font Functions
//=============================================================================

void SelectFont(unsigned char font_number)
{
	switch (font_number) {
	   case 0: currentfont = loaded_font;  // will be loadable font
	   		   break;
	   case 1: currentfont = Arial_Bold_14;
	   		   break;
	   case 2: currentfont = Corsiva_12;
	   		   break;
	   case 3: currentfont = Arial_24;
	   		   break;
	   default : currentfont = Arial_Bold_14;
	}
}
//-----------------------------------------------------------------------------
// Cursor
//-----------------------------------------------------------------------------
// cursor			: 0 -> Cursor off, 1 -> Cursor on
// cur_blink 	   : 0 -> static cursor, 1 -> blinking cursor
// cur_onoff		: 0 -> Cursor is not visible, 1 -> cursor is visible
// cur_type			: 0 -> underline, 1-> full
// cur_color		:
// cur_bgcolor		:
// cur_fix			:
// cur_landscape	:
// cur_cnt			: counter for blink
// cur_save[]     : 0 -> lower left corner ,
// coordx_save    :
// coordy_save    :
// CoordRAM_index_save :
//-----------------------------------------------------------------------------
void Cursor_Setup()
{
	unsigned char rpt = (currentfont[FONT_FIXED_WIDTH]+6)/3;
   unsigned char i,k;
   unsigned char x = Coordx, y = Coordy;
	Save_Current_Coord();
   if (cur_landscape) {
      if (cur_type) {    // landscape, full
			;
      }
		else {			     // landscape, underscore
         i=0;
			rpt = (rpt*2)+3;
			for (k=0;k<rpt;k++) { // read image for restore
			   cur_save[i++] = LCD_RAM_Image[CoordRAM_index++];// 2 pix (+1/3)
			   cur_save[i++] = LCD_RAM_Image[CoordRAM_index];	//
			   GotoXY(x,++y);
			}
			Restore_Current_Coord();
			Cursor_WR_Underscore_landscape ();
		}
	}
	else {
		if (cur_type) {     // upright, full
			;
		}
		else {			     // upright, underscore
          i=0;
			for (k=0;k<rpt;k++) { // read image for restore
			   cur_save[i++] = LCD_RAM_Image[CoordRAM_index+k];
			}
			GotoXY(x,y-1);
         for (k=0;k<rpt;k++) {
			   cur_save[i++] = LCD_RAM_Image[CoordRAM_index+k];
			}
			Restore_Current_Coord();
			Cursor_WR_Underscore_upright ();
		}
	}
   cursor = 1;
   Restore_Current_Coord();
}

void Cursor_WR_Underscore_upright()
{
unsigned char rpt = currentfont[FONT_FIXED_WIDTH];
unsigned char x,y,k;
	Save_Current_Coord();
	x = Coordx;
	y = Coordy;
	for (k=0;k<rpt;k++) {	  // write cursor  to image
	   SetDot(x+k,y,cur_color);
	   SetDot(x+k,y-1,cur_color);
	}
	Restore_Current_Coord();
   cur_onoff = 1;
}

void Cursor_WR_Underscore_landscape()
{
unsigned char rpt = currentfont[FONT_FIXED_WIDTH];
unsigned char x,y,k;
	Save_Current_Coord();
	x = Coordx;
	y = Coordy;
   for (k=0;k<rpt;k++) {
		SetDot(x,y+k,cur_color);
		SetDot(x+1,y+k,cur_color);
   }
	Restore_Current_Coord();
   cur_onoff = 1;
}

void Cur_Del_landscape()
{
unsigned char k,i,x,y;
unsigned char rpt = ((currentfont[FONT_FIXED_WIDTH]+6)/2)+3;
   i=0;    // restore image
	Save_Current_Coord();
	x = Coordx;
	y = Coordy;
   for (k=0;k<rpt;k++) { // restore image
	   LCD_RAM_Image[CoordRAM_index++] = cur_save[i++];
	   LCD_RAM_Image[CoordRAM_index] = cur_save[i++];
   GotoXY(x,++y);
   }
	Restore_Current_Coord();
   cur_onoff = 0;
}

void Cur_Del_upright()
{
unsigned char k,i,x,y;
unsigned char rpt = (currentfont[FONT_FIXED_WIDTH]+6)/3;
   i=0;    // restore image
	Save_Current_Coord();
	x = Coordx;
	y = Coordy;
	for (k=0;k<rpt;k++) {
	   LCD_RAM_Image[CoordRAM_index+k] = cur_save[i++];
	}
	GotoXY(x,y-1);
   for (k=0;k<rpt;k++) {
	   LCD_RAM_Image[CoordRAM_index+k] = cur_save[i++];
	}
	Restore_Current_Coord();
   cur_onoff = 0;
}

void Cur_clear(void)
{
	if (cursor && cur_onoff) {
	   if (cur_landscape) {
	      Cur_Del_landscape();
   	}
	   else {
		   Cur_Del_upright();
	   }
	}
}

void Save_Current_Coord(void)
{
	Coordx_save = Coordx;
	Coordy_save = Coordy;
	CoordRAM_index_save = CoordRAM_index ;
}

void Restore_Current_Coord(void)
{
	Coordx = Coordx_save;
	Coordy = Coordy_save;
	CoordRAM_index = CoordRAM_index_save ;
}

//-----------------------------------------------------------------------------
// write a char to image RAM
// bgcolor: (background color) f000 -> use current background - transparent
//
// fix: 0 -> not fix width, 1 -> fix width
// returns 1 if not valid char
// returns o if ok
//
// cluster is an horizontal area which contain 1 to 8 bit of a character
// cluster (cluster0) starts on top of the character
// below is cluster1, below cluster1 is cluster2 ....
//
// c is Character
//
//-----------------------------------------------------------------------------

int PutChar(unsigned char c, short color,short bgcolor, unsigned char fixed, unsigned char landscape)
{
unsigned char i,j,k,o,p;
unsigned char clusterindex;
unsigned char data;

	unsigned char real_width = 0;
	unsigned char char_height = currentfont[FONT_HEIGHT];
   unsigned char char_width = currentfont[FONT_FIXED_WIDTH];
	unsigned char cluster = (char_height+7)/8;

	unsigned char firstChar = currentfont[FONT_FIRST_CHAR];
	unsigned char charCount = currentfont[FONT_CHAR_COUNT];
   unsigned char x = Coordx, y = Coordy;

	unsigned int charindex = 0;
	unsigned char clusterx ,clustery ;

	Cur_clear();
	cur_landscape = landscape;
	cur_fix = fixed;

	if(c < firstChar || c >= (firstChar+charCount)) {
		return 1;
	}
	c-= firstChar;
	// read char_width data, to get the index
	for(i=0; i<c; i++) {
		charindex += currentfont[FONT_WIDTH_TABLE+i];
	}
	charindex = charindex*cluster+charCount+FONT_WIDTH_TABLE;
	real_width = currentfont[c+FONT_WIDTH_TABLE];
// last but not least, draw the character
   if (!landscape) {
	   for(i=0; i<cluster; i++) {
		   clusterindex = i*real_width;
		   clusterx = 0;
  		   clustery = char_height -(i*8) ;
		   if (clustery >= 8) {
		      o = 8;
		   }
		   else {
		      o = clustery;
		   }
		   for(j=0; j<real_width; j++) {      // display character
			   data = currentfont[charindex+clusterindex+j];
			   if(char_height < ((i+1)*8)) {
				   data >>= (i+1)*8-char_height;
			   }
		      clusterx ++;
			   for (k=0;k<o;k++) {
			      if (data & 0x01) {
	               SetDot(x+clusterx,y-clustery+k,color);
			      }
			      else {
			         if (!(bgcolor & 0xF000)) {
				         SetDot(x+clusterx,y-clustery+k,bgcolor);
				      }
			      }
			      data >>= 1;
			   }
		   }
// display 2px gap
		   clusterx ++;
		   for (k=0;k<o;k++) {
  	         if (!(bgcolor & 0xF000)) {
			      SetDot(x+clusterx,y+clustery-k,bgcolor);
		      }
		   }
		   clusterx ++;
		   for (k=0;k<o;k++) {
  	         if (!(bgcolor & 0xF000)) {
			      SetDot(x+clusterx,y+clustery-k,bgcolor);
		      }
		   }
// fill rest of char if fixed size
  		   if (fixed) {
		      for (p=0;p<char_width-j;p++) {
			      clusterx ++;
	              for (k=0;k<o;k++) {
  	                 if (!(bgcolor & 0xF000)) {
			              SetDot(x+clusterx,y+clustery-k,bgcolor);
		             }
		          }
		      }
		   }
	   }
//	next position
	   if (fixed) {
	      GotoXY(x+char_width+2, y);
	   }
	   else {
	      GotoXY(x+real_width+2, y);
	   }
	   if (cursor) {
         Cursor_Setup();
	   }
	   return 0;
   }
	else {	// landscape ---------------------------------------------
	   for(i=0; i<cluster; i++) {
		   clusterindex = i*real_width;
		   clusterx = char_height -(i*8);
         clustery = 0;
//		clustery = char_height -(i*8);
		   if (clusterx >= 8) {
		      o = 8;
		   }
		   else {
		      o = clusterx;
		   }
		   for(j=0; j<real_width; j++) {      // display character
			   data = currentfont[charindex+clusterindex+j];
			   if(char_height < (i+1)*8) {
				   data >>= (i+1)*8-char_height;
			   }
		      clustery ++;
			   for (k=0;k<o;k++) {
			      if (data & 0x01) {
	               SetDot(x+clusterx-k,y+clustery,color);
			      }
			      else {
			         if (!(bgcolor & 0xF000)) {
				         SetDot(x+clusterx-k,y+clustery,bgcolor);
				      }
			      }
			      data >>= 1;
			   }
		   }
// display 2px gap
		   clustery ++;
		   for (k=0;k<o;k++) {
  	         if (!(bgcolor & 0xF000)) {
			      SetDot(x+clusterx-k,y+clustery,bgcolor);
		      }
		   }
		   clustery ++;
		   for (k=0;k<o;k++) {
  	         if (!(bgcolor & 0xF000)) {
			      SetDot(x+clusterx-k,y+clustery,bgcolor);
		      }
		   }
// fill rest of char if fixed size
  		   if (fixed) {
		      for (p=0;p<char_width-j;p++) {
			      clustery ++;
	              for (k=0;k<o;k++) {
  	                 if (!(bgcolor & 0xF000)) {
			              SetDot(x+clusterx-k,y+clustery,bgcolor);
		             }
		          }
		      }
		   }
	   }
//	next position
	   if (fixed) {
	      GotoXY(x, y+char_width+2);
	   }
	   else {
	      GotoXY(x, y+real_width+2);
	   }
	   if (cursor) {
         Cursor_Setup();
	   }
	   return 0;
   }
}

void Puts(const unsigned char * str,short color,short bgcolor, unsigned char fix, unsigned char landscape) {
	int x = Coordx;
	while(*str != 0) {
		if(*str == '\n') {  // crlf
			GotoXY(x, Coordy+currentfont[FONT_HEIGHT]);
	      if (cursor) {
            Cursor_Setup();
	      }
		} else {
			PutChar(*str,color,bgcolor,fix,landscape);
		}
		str++;
	}
}

void Putss(unsigned char * str,short color,short bgcolor, unsigned char fix, unsigned char landscape) {
	int x = Coordx;
	while(*str != 0) {
		if(*str == '\n') {  // crlf
			GotoXY(x, Coordy+currentfont[FONT_HEIGHT]);
	      if (cursor) {
            Cursor_Setup();
	      }
		} else {
			PutChar(*str,color,bgcolor,fix,landscape);
		}
		str++;
	}
}

//--------------
// x = 0..175 (176/2*3) = 264 Byte
// y = 0..207
// total 54912 Byte
//   grrbbggr rbbggrrb bggrrbbg
//   00101122 32334454 55667677
// b     0        1        2

void GotoXY(unsigned char x, unsigned char y) {
unsigned char b;
	if(x > LCD_x_max) x = 0;			// ensure that coordinates are legal
	if(y > LCD_y_max) y = 0;
	Coordx = x;
	Coordy = y;
	Coordidx = (x & 0x00000007);
	switch (Coordidx) {
	   case 3:
	   case 4:
	   case 5: b=1;
	           break;
	   case 6:
	   case 7: b=2;
	           break;
	   default:b=0;
	}
	CoordRAM_index =  (x/8)*3+ b + (y*bytes_in_raw_x);
}

//-----------------------------------------------------------------------------
// set dot or dots
//-----------------------------------------------------------------------------
// width 0 : single dot
// width 1..x : 2*with+1 rectangular
//

void SetDotDot(unsigned char x, unsigned char y,unsigned char lwidth, unsigned short color)
{
   if (lwidth) {
	   x -= lwidth;
		y -= lwidth;
		lwidth *= 2;
		FillRect(x, y, lwidth,lwidth, 0, color);
	}
	else {
	   SetDot(x,y,color);
   }
}

//-----------------------------------------------------------------------------
// SetDot - Single Dot
// short color : 0x0rgb
// color : 12 bit
//           b0 b1 b2
//---------------------
//bit 31..28  r  b  g
//bit 27..24  g  r  b
//bit 23..20  g  r  b
//bit 19..16  b  g  r
//bit 15..12  b  g  r
//bit 11.. 8  r  b  g
//bit  7.. 4  r  b  g
//bit  3.. 0  g  r  b

//-----------------------------------------------------------------------------
void SetDot(unsigned char x, unsigned char y, unsigned short color)
{
	unsigned long dot;
	unsigned long r, g, b;
   r = (long)((color & 0x0F00)>>8);
   g = (long)((color & 0x00F0)>>4);
   b = (long)(color & 0x000F);
	GotoXY(x,y);
	dot = LCD_RAM_Image[CoordRAM_index];
	switch (Coordidx) {
       case 0 : //       ....b.rg
		 		dot &= 0xFFFF0F00;
                LCD_RAM_Image[CoordRAM_index] = dot | r<<4 | g | b<<12;
	             break;
	    case 1 : //       ..gb.r..
		 		dot &= 0xFF00F0FF;
                LCD_RAM_Image[CoordRAM_index] = dot | r<<8 | g<<20 | b<<16;
	             break;
	    case 2 : //       rg......
		 		  	 dot &= 0x00FFFFFF;
                LCD_RAM_Image[CoordRAM_index] = dot | r<<28 | g<<24 ;
	   		    dot = LCD_RAM_Image[++CoordRAM_index];
 					 //       ......b.
	             dot &= 0xFFFFFF0F;
                LCD_RAM_Image[CoordRAM_index] = dot | b<<4;;
	             break;
	    case 3 : //       ....gb.r
		 		  	 dot &= 0xFFFF00F0;
                LCD_RAM_Image[CoordRAM_index] = dot | r | g<<12 | b<<8;
	             break;
	    case 4 : //       b.rg....
		 		  	 dot &= 0x0F00FFFF;
                LCD_RAM_Image[CoordRAM_index] = dot | r<<20 | g<<16 | b<<28;
	             break;
	    case 5 : //       .r......
		 		  	 dot &= 0xF0FFFFFF;
                LCD_RAM_Image[CoordRAM_index] = dot | r<<24 ;
	   		    dot = LCD_RAM_Image[++CoordRAM_index];
	             //       ......gb
		 		  	 dot &= 0xFFFFFF00;
                LCD_RAM_Image[CoordRAM_index] = dot | g<<4 | b;
	             break;
	    case 6 : //       ..b.rg..
		 		  	 dot &= 0xFF0F00FF;
                LCD_RAM_Image[CoordRAM_index] = dot | r<<12 | g<<8 | b<<20;
	             break;
	    case 7 : //       gb.r....
		 		  	 dot &= 0x00F0FFFF;
                LCD_RAM_Image[CoordRAM_index] = dot |  r<<16 | g<<28 | b<<24;;
	             break;
	    default: ;
	}
}

unsigned long ReadRocker(void)
{
   Select_Source(1);      // read Rocker
   return((GPIO_IOPIN & 0x07F80000)>>19);
}

unsigned long ReadKey(void)
{
   Select_Source(2);      // read key
   return((GPIO_IOPIN & 0x07F80000)>>19);
}

//-----------------------------------------------------------------------------
// Write_Image_Line
// line : 0 ... 175
// short color  = 0rgb

void Write_Image_Line (unsigned char line, short color)
{
unsigned char i = 0;
unsigned int cnt;
   GotoXY(0,line);
   Translate_8Bit(color);
   cnt = ((LCD_x_max+1)/8);
   do  {
	  LCD_RAM_Image[CoordRAM_index++]= b0 ;
	  LCD_RAM_Image[CoordRAM_index++]= b1 ;
	  LCD_RAM_Image[CoordRAM_index++]= b2 ;
	  i++;
   }
   while(i < cnt);
}

//-----------------------------------------------------------------------------
//  UART0
//

void SendChar(unsigned char chr)
{
   while ((UART0_LSR & 0x60) != 0x60); // ULSR_THRE | ULSR_TEMT
	UART0_THR = chr;
}
void UART0_Start_Seq()
{
   SendChar(0xAA);
   SendChar(0x55);
}

void UART0_Stop_Seq()
{
   SendChar(0xF2);
}

void UnknownCommand(unsigned char command)
{
   UART0_Start_Seq();
   SendChar(0x02);
   SendChar(command);
   SendChar(0xF0);
   UART0_Stop_Seq();
}

void DC_Ready(unsigned char command)
{
   UART0_Start_Seq();
   SendChar(0x01);
   SendChar(command);
   UART0_Stop_Seq();
}

void Cmd_notyet_Available(unsigned char command)
{
   UART0_Start_Seq();
   SendChar(0x02);
   SendChar(command);
   SendChar(0xFF);
   UART0_Stop_Seq();
}

//*****************************************************************************
//---------------------------------------------------------------
// test different colors , direkt write
// Kann spter entfallen
void Reset_Test(void)
{
unsigned long k;
	  Write_LCD(0x5c,0x00);
     LCD_a0_high;
     for (k=0;k<4;k++)			      // 0--3
	  {
	     Write_Image_Line(k,0x0000);  //black
	  }
	  for (k=4;k<25;k++)			  // 4--24
	  {
	     Write_Image_Line(k,0x0F00);  //red
	  }
	  for (k=25;k<45;k++)			  // 25--44
	  {
	     Write_Image_Line(k,0x0F0);   //green
	  }
	  for (k=45;k<65;k++)			  // 45--64
	  {
		  Write_Image_Line(k,0x00F);  //blue
	  }
	  for (k=65;k<85;k++)			  // 65--84
	  {
		  Write_Image_Line(k,0x0FF0);  //red + green
	  }
	  for (k=85;k<105;k++)			  // 85--104
	  {
		  Write_Image_Line(k,0x0F70);  //red + 0.5 green
	  }
	  for (k=105;k<125;k++)			  // 105--124
	  {
		  Write_Image_Line(k,0x07F0);  //0.5 red + green
	  }
	  for (k=125;k<145;k++)			  // 125--144
	  {
		  Write_Image_Line(k,0x0F0F);  //red + blue
	  }
	  for (k=145;k<165;k++)			  // 145--164
	  {
		  Write_Image_Line(k,0x0F07);  //red + 0.5 blue
	  }
	  for (k=165;k<185;k++)			  // 165--184
	  {
		  Write_Image_Line(k,0x070F);  //0.5 red + blue
	  }
	  for (k=185;k<205;k++)			  // 185--204
	  {
		  Write_Image_Line(k,0x0FF);  //green + blue
	  }
     for (k=205;k<209;k++)			      // 205--208
	  {
		  Write_Image_Line(k,0x0FFF);	  //white
	  }
}

// ende test different colors

void System_ID(void)
{
 	   GotoXY(100,10);
      Puts(teststring0,0x0,0xf000,0,1);
 	   GotoXY(60,60);
      Puts(teststring1,0x0,0xf000,0,1);
}

void System_Message(void)
{
	   Reset_Test();
		System_ID();
	   Image2lcdasm();
}

//-----------------------------------------------------------------------------
// main program
//-----------------------------------------------------------------------------
int main(void)
{

unsigned char lcd_a0,len;
unsigned long j,l,i,z;
unsigned char a,b,c,ba,ac,cb ;
unsigned short bgcolor,color;
unsigned char text[16];

	   init_devices();
      Select_Source(0);
		rocker_value = 0;
		key_value = 0;
//-------------------------------------------------


//-------------------------------------------------
// Prepare Display
       LCD_wr_high;
       LCD_rd_high;
       LCD_rst_high;
       wait(100);			//  100 ms

	    LCD_rst_low;
       wait(170);			// 170 ms
       LCD_rst_high;
       wait(50);			// 50 ms

//-------------------------------------------------------
// First Init and check which display is used
   	 Write_LCD(0x94,0x00);
       wait(100);			// 100 ms
	    Write_LCD(0x00,0x00);
// read
       wait(50);			// 50ms
//	   lcd_data = Read_LCD(0);
// wait
//       wait(100);			//

//-------------------------------------------------------
// Second Init
	    LCD_rst_low;
       wait(170);			// 170 ms
       LCD_rst_high;
       wait(50);			// 50ms
//-------------------------------------------------------

	   Write_LCD(0xCB,0x00);
	   Write_LCD(0x13,0x01);
	   Write_LCD(0x24,0x01);
	   Write_LCD(0x31,0x01);
	   Write_LCD(0x39,0x01);
	   Write_LCD(0x40,0x01);
	   Write_LCD(0x45,0x01);
	   Write_LCD(0x4A,0x01);
	   Write_LCD(0x4D,0x01);
	   Write_LCD(0x50,0x01);
	   Write_LCD(0x53,0x01);
	   Write_LCD(0x56,0x01);
	   Write_LCD(0x59,0x01);
	   Write_LCD(0x5B,0x01);
	   Write_LCD(0x5E,0x01);
	   Write_LCD(0x78,0x01);

//-------------------------------------------------------

	   Write_LCD(0xCA,0x00);
	   Write_LCD(0x63,0x01);
	   Write_LCD(0x00,0x01);
	   Write_LCD(0x1E,0x01);
	   Write_LCD(0x00,0x01);
	   Write_LCD(0x69,0x01);
	   Write_LCD(0x67,0x01);
	   Write_LCD(0x01,0x01);
//-------------------------------------------------------

	   Write_LCD(0x15,0x00);
	   Write_LCD(0xC0,0x01);
	   Write_LCD(0x00,0x01);
	   Write_LCD(0xC7,0x01);
	   Write_LCD(0x01,0x01);
//-------------------------------------------------------

	   Write_LCD(0xBC,0x00);
	   Write_LCD(0x0A,0x01);
//-------------------------------------------------------

	   Write_LCD(0xA6,0x00);
//-------------------------------------------------------

	   Write_LCD(0x94,0x00);
//-------------------------------------------------------

	   Write_LCD(0xC6,0x00);
	   Write_LCD(0x98,0x01);
	   wait(200); // 200ms
//-------------------------------------------------------

	   Write_LCD(0xC6,0x00);
	   Write_LCD(0x00,0x01);
//-------------------------------------------------------

//	   Write_LCD(0xA9,0x00);
//-------------------------------------------------------

	   Write_LCD(0x75,0x00);
	   Write_LCD(0x00,0x01);
 	   Write_LCD(0xD0,0x01);

  	   Select_Source(0);
	   wait(50);			// 50ms
  	   SelectFont(3);
		System_Message();

		cur_color = 0x000; // black
		cur_bgcolor = 000;
		cur_blink = 1;
		cur_type = 0;
		cur_onoff = 0;
		cursor = 0;
		cur_fix = 0;
		cur_landscape = 0;

	   while(1)
	   {

//----------------------------------------------------------
       process =0;
		 switch (rx_rdy) {
          case 5: process = 1;
			 		   rx_in = 0;
	   		      break;        // message ready and ok
	       case 6: UART0_Start_Seq();  // error ??!!
					   SendChar(0x01);
					   SendChar(0xFF);
						UART0_Stop_Seq();
			 		   rx_in = 0;
						rx_rdy = 0;
	   		      break;		  // message ready and not ok
	       default: ;
       }
		 if (process) {
		 	 command = Rx_Buff[0];
			 if (command < 0x10) {
//			 	  0..0F;
			    switch (command) {
				    case 0: DC_Ready(command); 		 	  		//Nop
					         break;
				    case 1: L2F_DisplayNormal;					// Reset
					         System_Message();					   //---
					         DC_Ready(command);
								break;
				    case 2: System_ID();							//---
					 			Image2lcdasm();
					 			DC_Ready(command);
					         break;
				    case 3: L2F_DisplayOn;					       //---
					         DC_Ready(command);
					         break;
				    case 4: L2F_DisplayOff;					    //---
					         DC_Ready(command);
					         break;
					 case 5: L2F_DisplayNormal;					 //---
					 		   DC_Ready(command);
					         break;
					 case 6: L2F_DisplayInvers;					 //---
					 		   DC_Ready(command);
					         break;
					 case 7: Back_Light(Rx_Buff[1]);				 //---
					         DC_Ready(command);
					         break;
					 case 8: Back_Light_UD(1);
					         DC_Ready(command);
					         break;
					 case 9: Back_Light_UD(0);
					         DC_Ready(command);
					         break;
					 case 10:color = (short)(Rx_Buff[1]) + ((short)(Rx_Buff[2])<<8);
					         L2F_Background(color);
					 			Image2lcdasm();
								DC_Ready(command);
					         break;
					 case 11:Cmd_notyet_Available(command);
					         break;
					 case 12:Cmd_notyet_Available(command);
					         break;
					 case 14:Copy_Image_FR();
					 		   GotoXY(84,80);
								SelectFont(1);
								Puts(tbm,0xfff,0xf000,0,1);
					         Image2lcdasm();
	 			 			   DC_Ready(command);;
					         break;
					 case 15: Cmd_notyet_Available(command);
					         break;
					 default: UnknownCommand(command);
				 }
			 }
			 else {
			    if (command < 0x20) {
//			 	     10..1F;
				    switch(command) {
					 case 16:Image2lcdasm();
	 			 			   DC_Ready(command);
								break;
					 case 18:color = (short)(Rx_Buff[3]) + ((short)(Rx_Buff[4])<<8);
								SetDot(Rx_Buff[1],Rx_Buff[2],color);
					 			Image2lcdasm();
					         DC_Ready(command);
								break;
					 case 19:color = (short)(Rx_Buff[2]) + ((short)(Rx_Buff[3])<<8);
					 			Write_Image_Line(Rx_Buff[1],color);
					 			Image2lcdasm();
					         DC_Ready(command);
								break;
					 case 21:SelectFont(Rx_Buff[1]);
					         DC_Ready(command);
					         break;
					 case 22:Cur_clear();
					 			GotoXY(Rx_Buff[1],Rx_Buff[2]);
					         if (cursor) {
								   Cursor_Setup();
								}
					         DC_Ready(command);
					         break;
					 case 23:color = (short)(Rx_Buff[2]) + ((short)(Rx_Buff[3])<<8);
					         bgcolor = (short)(Rx_Buff[4]) + ((short)(Rx_Buff[5])<<8);
					         PutChar(Rx_Buff[1],color,bgcolor, Rx_Buff[6], Rx_Buff[7]) ;
					 			Image2lcdasm();
					         DC_Ready(command);
					         break;
					 case 24:
					 			len = lastlen;
					 			color = (short)(Rx_Buff[len-6]) + ((short)(Rx_Buff[len-5])<<8);
					         bgcolor = (short)(Rx_Buff[len-4]) + ((short)(Rx_Buff[len-3])<<8);
					         for (i=1;i<len-6;i++) {
								   text[i-1] = (Rx_Buff[i]) ;
					 			}
								text[i-1]= 0;
								Putss(text,color,bgcolor, Rx_Buff[len-2], Rx_Buff[len-1]);
								Image2lcdasm();
					         DC_Ready(command);
					         break;
					 case 25:color = (short)(Rx_Buff[5]) + ((short)(Rx_Buff[6])<<8);
								DrawCircle(Rx_Buff[1],Rx_Buff[2], Rx_Buff[3],Rx_Buff[4], color);
								Image2lcdasm();
					         DC_Ready(command);
					         break;
					 case 26:color = (short)(Rx_Buff[6]) + ((short)(Rx_Buff[7])<<8);
								DrawLine(Rx_Buff[1],Rx_Buff[2], Rx_Buff[3],Rx_Buff[4],Rx_Buff[5], color);
								Image2lcdasm();
					         DC_Ready(command);
					         break;
					 case 27:color = (short)(Rx_Buff[6]) + ((short)(Rx_Buff[7])<<8);
								DrawRect(Rx_Buff[1],Rx_Buff[2],Rx_Buff[3],Rx_Buff[4],Rx_Buff[5], color);
								Image2lcdasm();
					         DC_Ready(command);
					         break;
					 case 28:color = (short)(Rx_Buff[7]) + ((short)(Rx_Buff[8])<<8);
								DrawRoundRect(Rx_Buff[1],Rx_Buff[2],Rx_Buff[3],Rx_Buff[4],Rx_Buff[5],Rx_Buff[6], color);
								Image2lcdasm();
					         DC_Ready(command); ;
					         break;
					 case 29:color = (short)(Rx_Buff[6]) + ((short)(Rx_Buff[7])<<8);
								FillRect(Rx_Buff[1],Rx_Buff[2],Rx_Buff[3],Rx_Buff[4],Rx_Buff[5], color);
								Image2lcdasm();
					         DC_Ready(command);
					         break;
					 case 30:cur_type = (Rx_Buff[1]);
					         if (cur_type) {
								   cursor = 1;
									cur_type--;
								   cur_color = (short)(Rx_Buff[2]) + ((short)(Rx_Buff[3])<<8);
								   cur_bgcolor = (short)(Rx_Buff[4]) + ((short)(Rx_Buff[5])<<8);
                           cur_blink = Rx_Buff[6];
								   Cursor_Setup();
								}
								else {
									cursor = 0;
									cur_blink = 0;
					    			if (cur_landscape) {
						    			Cur_Del_landscape();
						 			}
						 			else {
						    			Cur_Del_upright();
						 			}
							   }
								Image2lcdasm();
					         DC_Ready(command);
					         break;
				    default: UnknownCommand(command);
				    }
				 }
				 else {
				    if (command < 0x30) {
//			 	        20..2F;
				       switch(command) {
					       default: UnknownCommand(command);
				       }
					 }
					 else {
				       switch(command) {
					       default: UnknownCommand(command);
				       }
					 }
				 }
			 }
		    rx_rdy = 0;
		 }
       switch (rocker_value & 0xE0000000) {
		    case 0x80000000 : wait(debounce);		  // debounce close
                            i = rocker_value;
			                   rocker_value = ReadRocker() | 0xC0000000;
									 Select_Source(0);
									 if (i == rocker_value) {
									    ; // action
									 }
									 break;
	       case 0xC0000000 : j = ReadRocker();
			 					  	 if ((j & 0x0000001F) == 0x0000001F) {
									    Select_Source(0);
			                      wait(debounce);   // debounce open
         							 SCB_EXTINT |= 0x01; // Clear EINT0 interrupt flag
										 rocker_value = 0;
										 VICIntEnable=0x00004000;
                            }
									 break;
			default : 			 ;
		 }
		 if (key_value && 0x80000000) {
		    wait(5);
		 }
		 if (cursor) {
		    if (cur_blink ) {
			    z = T0_TC;
			    if (z - cur_cnt > cur_blinkfreq) {
				    cur_cnt = T0_TC;
					 if (cur_onoff) {					 // turn cursor off
					 	 Cur_clear();
					    Image2lcdasm();
					 }
					 else {   	  	 	 	 			 // turn cursor on
					    if (cur_landscape) {
						    Cursor_WR_Underscore_landscape ();
						 }
						 else {
						    Cursor_WR_Underscore_upright ();
						 }
						 Image2lcdasm();
					 }
				 }
			 }
		 }
	}
	return 0;
}

