/*----------------------------------------------------------------------------
    General definitions for NOKIA7650
-----------------------------------------------------------------------------*/

// LPC2106 IO

// P0.00	s	TXD0	: 13
// P0.01	s	RXD0	: 14
// P0.02	s	SLC		: 18
// P0.03	s	SDA		: 21

// P0.04	s	SCK		: 22
// P0.05	s	MISO	: 23
// P0.06	s	MOSI	: 24
// P0.07	s	SSEL	: 28

// P0.08	ss	LEDPWM	: 29  
// P0.09	o	RESET 	: 30
// P0.10	o	OELCD 	: 35  
// P0.11	o	SEL0 	: 36  

// P0.12	o	SEL1	: 37  
// P0.13	o 	test	: 41
// P0.14	s  	int1	: 44
// P0.15	o  	A0   	: 45 

// P0.16	s  	int0	: 46
// P0.17	o	WR      : 47
// P0.18	o	RD		: 48
// P0.19	o	D0		: 1

// P0.20	o	D1		: 2
// P0.21	o	D2		: 3
// P0.22	o	D3 		: 32
// P0.23	o	D4 		: 33

// P0.24	o	D5 		: 34
// P0.25	o	D6 		: 38
// P0.26	o	D7 		: 39
// P0.27	s	TRST  	: 8

// P0.28	s	TMS 	: 9
// P0.29	s	TCK 	: 10
// P0.30	s	TDI 	: 15
// P0.31	s	TDO 	: 16

/* DEFINE IOMASK */

#define NULL            0
#define FALSE           0
#define TRUE            1

#define  LCD_D0  0x00080000    
#define  LCD_D1  0x00100000    
#define  LCD_D2  0x00200000    
#define  LCD_D3  0x00400000    
#define  LCD_D4  0x00800000    
#define  LCD_D5  0x01000000   
#define  LCD_D6  0x02000000    
#define  LCD_D7  0x04000000   
#define  LCD_Wr  0x00020000  
#define  LCD_Rd  0x00040000  
#define  LCD_Rst 0x00000200   
#define  LCD_A0  0x00008000   

#define  LCD_Sel0  0x00000800  
#define  LCD_Sel1  0x00001000   
#define  LCD_OELCD 0x00000400   

#define  test    0x00002000  

#define  LCD_DATA   (LCD_D7|LCD_D6|LCD_D5|LCD_D4|LCD_D3|LCD_D2|LCD_D1|LCD_D0)
#define  LCD_IOALL  (test|LCD_Wr|LCD_Rd|LCD_Rst|LCD_A0|LCD_D7|LCD_D6|LCD_D5|LCD_D4|LCD_D3|LCD_D2|LCD_D1|LCD_D0)

#define  PINSEL0 0x01025555    
#define  PINSEL1 0x55400001  


#define IODIR 0x07FEBF00   
// for data read
#define IODIR1 0x006BF00 

#define IOCLR 0xFFFFFFFF
#define IOSET 0x80000000

/* internal I/O functions */
#define LCD_wr_high GPIO_IOSET = LCD_Wr
#define LCD_wr_low GPIO_IOCLR = LCD_Wr
#define LCD_rd_high GPIO_IOSET = LCD_Rd
#define LCD_rd_low GPIO_IOCLR = LCD_Rd
#define LCD_rst_high GPIO_IOSET = LCD_Rst
#define LCD_rst_low GPIO_IOCLR = LCD_Rst
#define LCD_a0_high GPIO_IOSET = LCD_A0
#define LCD_a0_low GPIO_IOCLR = LCD_A0

#define LCD_SEL0_high GPIO_IOSET = LCD_Sel0
#define LCD_SEL0_low GPIO_IOCLR = LCD_Sel0
#define LCD_SEL1_high GPIO_IOSET = LCD_Sel1
#define LCD_SEL1_low GPIO_IOCLR = LCD_Sel1
#define LCD_OELCD_high GPIO_IOSET = LCD_OELCD
#define LCD_OELCD_low GPIO_IOCLR = LCD_OELCD

#define test_high GPIO_IOSET = test
#define test_low GPIO_IOCLR = test

#define LCD_x_max 175
#define LCD_y_max 207
#define bytes_in_raw_x (LCD_x_max+1)/2*3/4


/* 
	PLL
	
   - Main clock F_OSC=14,7MHz @ Olimex LPC-P2106) [limits: 10 MHz to 25 MHz]
   - System should run at max. Frequency (60MHz) [limit: max 60 MHz]
   - Choose multiplier M=4 so cclk = M * F_OSC= 4 * 14745600Hz = 58980000 Hz
   - MSEL-Bit in PLLCFG (bits 0-4) MSEL = M-1
   - F_CCO must be inbetween the limits 156 MHz to 320 MHz
     datasheet: F_CCO = F_OSC * M * 2 * P
   - choose devider P=2 => F_CCO = 14745600Hz * 4 * 2 * 2 = 235920000 ~=236 MHz
   - PSEL0 (Bit5 in PLLCFG) = 1, PSEL1 (Bit6) = 0 (0b01)
*/
#define FOSC		14745600
#define PLL_M		4	
#define MSEL		(PLL_M-1)
#define PSEL0 		5
#define PSEL1 		6

#define PLLE		0
#define PLLC		1

#define PLOCK		10

#define PLL_FEED1	0xAA
#define PLL_FEED2	0x55

//-------------------------------------------------------
// Data out
//
 
#define DataClear_rr GPIO_IOCLR = LCD_DATA
#define DataOut_rr   GPIO_IOSET = rr << 19
#define DataClear_gg GPIO_IOCLR = LCD_DATA
#define DataOut_gg   GPIO_IOSET = gg << 19
#define DataClear_bb GPIO_IOCLR = LCD_DATA
#define DataOut_bb   GPIO_IOSET = bb << 19

// PWM setup
// around 1 ms  periode
#define pwm_periode 0x00004000	


//****************************************************************************
// Font Indices

// Font Indices
#define FONT_LENGTH			0
#define FONT_FIXED_WIDTH	2
#define FONT_HEIGHT			3
#define FONT_FIRST_CHAR		4
#define FONT_CHAR_COUNT		5
#define FONT_WIDTH_TABLE	6

#define DrawVertLine(x, y, length, lwidth, color) {FillRect(x, y, 0, length, lwidth, color);}
#define DrawHoriLine(x, y, length, lwidth, color) {FillRect(x, y, length, 0, lwidth, color);}
#define DrawCircle(xCenter, yCenter, radius, lwidth, color) {DrawRoundRect(xCenter-radius, yCenter-radius, 2*radius, 2*radius, radius, lwidth, color);}


